﻿/**
 * CIS 399-005: C# Programming (Spring 2009)
 * Problem Set 4 - ElevatorPlanner.dll
 */

/// <summary>
/// Named constants for the elevator simulation.
/// </summary>
public sealed class ElevatorConstants
{
    public const int NumFloors = 5;
    public const int NumElevators = 2;
    public const int ElevatorCapacity = 5;
}

/// <summary>
/// Plans the actions for a set of elevators.
/// </summary>
public interface IElevatorPlanner
{
    /// <summary>
    /// Plans a step for the set of elevators.
    /// </summary>
    /// <param name="controls">
    /// The set of controls where controls[i] corresponds to the ith elevator.
    /// </param>
    /// <param name="upFloors">
    /// UpFloors[i] == true iff a sim on floor i is in state GoingTo(j) with
    /// j > i.
    /// </param>
    /// <param name="downFloors">
    /// downFloors[i] == true iff a sim on floor i is in state GoingTo(j) with
    /// j < i.
    /// </param>
    void PlanStep(IElevatorControl[] controls,
                  bool[] upFloors,
                  bool[] downFloors);
}

/// <summary>
/// The set of actions an elevator takes.
/// </summary>
public enum ElevAction { Open, Stopped, GoingUp, GoingDown };

/// <summary>
/// The set of states of an elevator's light.
/// </summary>
public enum ElevLight { Up, Down, None };

/// <summary>
/// Provides information about and sets the actions of a single elevator.
/// </summary>
public interface IElevatorControl {

    /// <param name="i">The floor in question.</param>
    /// <returns>
    /// True iff some sim in the associated elevator is in state GoingTo(i).
    /// </returns>
    bool StopRequested(int i);

    /// <summary>
    /// The elevator's current state.
    /// </summary>
    ElevAction ActionState { get; }

    /// <summary>
    /// Sets the elevator's state, ensuring that the action is a valid
    /// transition.
    /// </summary>
    /// <param name="s">The action the elevator should take.</param>
    void SetAction(ElevAction s);

    /// <summary>
    /// The state of the elevator's light.
    /// </summary>
    ElevLight Indicator { get; set; }

    /// <summary>
    /// The height of the elevator.
    /// </summary>
    int Height { get; }
}
